Imports System
Imports System.Collections.Generic
Imports System.Text
Imports System.Runtime.InteropServices
Imports System.IO

<ComVisible(False)> _
Public Class GlobalData

    Private Shared _instance As GlobalData

    Private lastOutputPath As String
    Private optionPageData As MySharedData

    Private Const FILE_LOCKED As Integer = 0
    Private Const RADIO_OPTION As Integer = 1
    Private Const CHECK_OPTION_1 As Integer = 2
    Private Const CHECK_OPTION_2 As Integer = 3
    Private Const LIST_OPTION As Integer = 4
    Private Const STRING_OPTION As Integer = 5

    Public Sub New()
        MyBase.New()
        'System.Diagnostics.Debugger.Break()
        Dim outputPath As String = _
            Path.Combine(Environment.GetFolderPath( _
            Environment.SpecialFolder.InternetCache), "Low")
        outputPath = Path.Combine(outputPath, "MyIEOptionPage")
        If Not Directory.Exists(outputPath) Then
            Directory.CreateDirectory(outputPath)
        End If
        Me.lastOutputPath = outputPath
        Me.optionPageData = New MySharedData( _
            "MyIEOptionPage", 512, 10)

        Me.optionPageData(FILE_LOCKED) = 0
        Me.optionPageData(RADIO_OPTION) = 0
        Me.optionPageData(CHECK_OPTION_1) = False
        Me.optionPageData(CHECK_OPTION_2) = False
        Me.optionPageData(LIST_OPTION) = 0
        Me.optionPageData(STRING_OPTION) = "Default"
    End Sub

    Public Sub ConnectOptions()
        If Me.optionPageData Is Nothing Then
            Me.optionPageData = New MySharedData("MyIEOptionPage", 512, 10)

            Me.optionPageData(FILE_LOCKED) = 0
            Me.optionPageData(RADIO_OPTION) = 0
            Me.optionPageData(CHECK_OPTION_1) = False
            Me.optionPageData(CHECK_OPTION_2) = False
            Me.optionPageData(LIST_OPTION) = 0
            Me.optionPageData(STRING_OPTION) = "Default"
        End If
    End Sub

    Public Shared ReadOnly Property Instance() As GlobalData
        Get
            If (GlobalData._instance Is Nothing) Then
                GlobalData._instance = New GlobalData()
            End If
            Return GlobalData._instance
        End Get
    End Property

    Public Sub LoadOptions()
        ConnectOptions()

        If CInt(Me.optionPageData(FILE_LOCKED)) = 0 Then
            Me.optionPageData(FILE_LOCKED) = 1
            Try
                Dim outputFile As String = _
                    Path.Combine(lastOutputPath, "myoptions.cfg")
                If File.Exists(outputFile) Then
                    Using sr As New StreamReader(outputFile)
                        Dim line As String
                        line = sr.ReadLine()
                        Me.optionPageData(RADIO_OPTION) = _
                          Convert.ToInt32(line.Trim())
                        line = sr.ReadLine()
                        Me.optionPageData(CHECK_OPTION_1) = _
                          Convert.ToBoolean(line.Trim())
                        line = sr.ReadLine()
                        Me.optionPageData(CHECK_OPTION_2) = _
                          Convert.ToBoolean(line.Trim())
                        line = sr.ReadLine()
                        Me.optionPageData(LIST_OPTION) = _
                          Convert.ToInt32(line.Trim())
                        line = sr.ReadLine()
                        Me.optionPageData(STRING_OPTION) = _
                          line.Trim()
                    End Using
                End If
            Finally
                Me.optionPageData(FILE_LOCKED) = 0
            End Try
        End If
    End Sub

    Private Sub SaveOptions()
        If CInt(Me.optionPageData(FILE_LOCKED)) = 0 Then
            Me.optionPageData(FILE_LOCKED) = 1
            Try
                Dim outputFile As String = _
                    Path.Combine(lastOutputPath, "myoptions.cfg")
                Using sw As New StreamWriter(outputFile, False)
                    sw.Write(Me.optionPageData(RADIO_OPTION))
                    sw.Write(sw.NewLine)
                    sw.Write(Me.optionPageData(CHECK_OPTION_1))
                    sw.Write(sw.NewLine)
                    sw.Write(Me.optionPageData(CHECK_OPTION_2))
                    sw.Write(sw.NewLine)
                    sw.Write(Me.optionPageData(LIST_OPTION))
                    sw.Write(sw.NewLine)
                    sw.Write(Me.optionPageData(STRING_OPTION))
                End Using
            Finally
                Me.optionPageData(FILE_LOCKED) = 0
            End Try
        End If
    End Sub

    Public Sub UpdateOptions(ByVal [module] As IEBarModule)
        ConnectOptions()

        SyncLock Instance
            Me.optionPageData(RADIO_OPTION) = [module].RadioOptions
            Me.optionPageData(CHECK_OPTION_1) = [module].CheckOption1
            Me.optionPageData(CHECK_OPTION_2) = [module].CheckOption2
            Me.optionPageData(LIST_OPTION) = [module].ListOption
            Me.optionPageData(STRING_OPTION) = [module].StringOption
            SaveOptions()
        End SyncLock
    End Sub

    Public Sub RefreshOptions(ByVal [module] As IEBarModule)
        ConnectOptions()

        SyncLock Instance
            [module].RadioOptions = _
              CInt(Me.optionPageData(RADIO_OPTION))
            [module].CheckOption1 = _
              CBool(Me.optionPageData(CHECK_OPTION_1))
            [module].CheckOption2 = _
              CBool(Me.optionPageData(CHECK_OPTION_2))
            [module].ListOption = _
              CInt(Me.optionPageData(LIST_OPTION))
            [module].StringOption = _
              DirectCast(Me.optionPageData(STRING_OPTION), String)
        End SyncLock
    End Sub

    Public Sub CloseOptions()
        SyncLock Instance
            If Me.optionPageData IsNot Nothing Then
                Try
                    SaveOptions()
                Finally
                    Me.optionPageData.Dispose()
                    Me.optionPageData = Nothing
                End Try
            End If
        End SyncLock
    End Sub
End Class
